package net.knarcraft.armoredelytra.metadata;

import net.knarcraft.armoredelytra.container.EnchantmentContainer;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.io.Serializable;
import java.util.List;

/**
 * An object for storing the original metadata for an armored elytra.
 *
 * <p>The intended usage of this class is to keep track of the original metadata of the armor and the elytra for each
 * armored elytra, to prevent problems when splitting them. If an armor used to upgrade the armored elytra has a custom
 * name or lore, and none is stored already, the name and lore should be updated. Additionally, any enchantments found
 * on armor used to upgrade the armored elytra should be combined with the original armor enchantments.</p>
 */
public class OriginalMetadata implements Serializable {

    private EnchantmentContainer armorEnchantments = new EnchantmentContainer();
    private EnchantmentContainer elytraEnchantments = new EnchantmentContainer();
    private List<String> elytraLore;
    private String elytraName;
    private List<String> armorLore;
    private String armorName;

    public OriginalMetadata() {

    }

    /**
     * Gets the armor enchantments stored in this metadata.
     *
     * @return <p>The stored armor enchantments.</p>
     */
    public @NotNull EnchantmentContainer getArmorEnchantments() {
        return armorEnchantments;
    }

    /**
     * Gets the elytra enchantments stored in this metadata.
     *
     * @return <p>The stored elytra enchantments.</p>
     */
    public @NotNull EnchantmentContainer getElytraEnchantments() {
        return elytraEnchantments;
    }

    /**
     * Gets the original lore of this elytra.
     *
     * @return <p>The original elytra lore.</p>
     */
    public @Nullable List<String> getElytraLore() {
        return this.elytraLore;
    }

    /**
     * Gets the original name of this elytra.
     *
     * @return <p>The original elytra name.</p>
     */
    public @Nullable String getElytraName() {
        return this.elytraName;
    }

    /**
     * Gets the original lore of this armor.
     *
     * @return <p>The original armor lore.</p>
     */
    public @Nullable List<String> getArmorLore() {
        return this.armorLore;
    }

    /**
     * Gets the original name of this armor.
     *
     * @return <p>The original armor name.</p>
     */
    public @Nullable String getArmorName() {
        return this.armorName;
    }

    /**
     * Sets the stored lore of this armored elytra's armor.
     *
     * @param armorLore <p>The armor lore to store.</p>
     */
    public void setArmorLore(@NotNull List<String> armorLore) {
        this.armorLore = armorLore;
    }

    /**
     * Sets the stored elytra lore of this armored elytra's elytra.
     *
     * @param elytraLore <p>The elytra lore to store.</p>
     */
    public void setElytraLore(@NotNull List<String> elytraLore) {
        this.elytraLore = elytraLore;
    }

    /**
     * Sets the stored elytra name of this armored elytra's elytra.
     *
     * @param elytraName <p>The elytra name to store.</p>
     */
    public void setElytraName(@NotNull String elytraName) {
        this.elytraName = elytraName;
    }

    /**
     * Sets the stored armor name of this armored elytra's armor.
     *
     * @param armorName <p>The armor name to store.</p>
     */
    public void setArmorName(@NotNull String armorName) {
        this.armorName = armorName;
    }

    /**
     * Sets the stored enchantments for this armored elytra's armor.
     *
     * @param armorEnchantments <p>The armor enchantments to store.</p>
     */
    public void setArmorEnchantments(@NotNull EnchantmentContainer armorEnchantments) {
        this.armorEnchantments = armorEnchantments;
    }

    /**
     * Sets the stored enchantments for this armored elytra's elytra.
     *
     * @param elytraEnchantments <p>The elytra enchantments to store.</p>
     */
    public void setElytraEnchantments(@NotNull EnchantmentContainer elytraEnchantments) {
        this.elytraEnchantments = elytraEnchantments;
    }

}
