package net.knarcraft.armoredelytra.metadata.builder;

import net.knarcraft.armoredelytra.ArmoredElytra;
import net.knarcraft.armoredelytra.container.EnchantmentContainer;
import net.knarcraft.armoredelytra.metadata.NBTEditor;
import net.knarcraft.armoredelytra.metadata.OriginalMetadata;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;

import java.util.Objects;
import java.util.function.Predicate;

/**
 * A class for building the combined original metadata for the given items.
 */
public class OriginalMetadataBuilder {

    private final ArmoredElytra plugin;
    private final NBTEditor nbtEditor;

    /**
     * Instantiates a new original metadata builder
     *
     * @param plugin    <p>A reference to the plugin class.</p>
     * @param nbtEditor <p>A reference to the NBT editor</p>
     */
    public OriginalMetadataBuilder(ArmoredElytra plugin, NBTEditor nbtEditor) {
        this.plugin = plugin;
        this.nbtEditor = nbtEditor;
    }

    /**
     * Gets the merged original metadata for two items.
     *
     * @param elytra <p>The input (armored) elytra.</p>
     * @param armor  <p>The armored elytra or chestplate to be combined with the elytra.</p>
     * @return <p>The combined original metadata for the two items.</p>
     */
    public OriginalMetadata getMergedOriginalMetaData(ItemStack elytra, ItemStack armor) {
        OriginalMetadata elytraMetadata = nbtEditor.getOriginalMetadata(elytra);
        OriginalMetadata armorMetadata = nbtEditor.getOriginalMetadata(armor);

        // If metadata is missing from one of the items, create new metadata
        if (elytraMetadata == null) {
            elytraMetadata = createElytraMetadata(elytra);
        }
        if (armorMetadata == null) {
            armorMetadata = createArmorMetadata(armor);
        }

        // Merge elytra and armor enchantments
        elytraMetadata.getElytraEnchantments().merge(armorMetadata.getElytraEnchantments());
        elytraMetadata.getArmorEnchantments().merge(armorMetadata.getArmorEnchantments());

        // Use the elytra name from the armor if not set for the elytra
        if (useSecondValue(elytraMetadata.getElytraName(), armorMetadata.getElytraName(), (item) -> !item.isBlank())) {
            elytraMetadata.setElytraName(Objects.requireNonNull(armorMetadata.getElytraName()));
        }

        // Use the armor name from the armor if not set for the elytra
        if (useSecondValue(elytraMetadata.getArmorName(), armorMetadata.getArmorName(), (item) -> !item.isBlank())) {
            elytraMetadata.setArmorName(Objects.requireNonNull(armorMetadata.getArmorName()));
        }

        // Use the elytra lore from the armor if not set for the elytra
        if (useSecondValue(elytraMetadata.getElytraLore(), armorMetadata.getElytraLore(), (item) -> !item.isEmpty())) {
            elytraMetadata.setElytraLore(Objects.requireNonNull(armorMetadata.getElytraLore()));
        }

        // Use the armor lore from the armor if not set for the elytra
        if (useSecondValue(elytraMetadata.getArmorLore(), armorMetadata.getArmorLore(), (item) -> !item.isEmpty())) {
            elytraMetadata.setArmorLore(Objects.requireNonNull(armorMetadata.getArmorLore()));
        }

        return elytraMetadata;
    }

    /**
     * A test to see if the second value given matches the predicate, and the first one does not.
     *
     * @param value1    <p>The first value to test.</p>
     * @param value2    <p>The second value to test.</p>
     * @param predicate <p>The predicate to run.</p>
     * @param <T>       <p>The type of the values.</p>
     * @return <p>True if the second value matches the predicate, but the first value does not.</p>
     */
    private <T> boolean useSecondValue(T value1, T value2, Predicate<T> predicate) {
        return (value1 == null || !predicate.test(value1)) && (value2 != null && predicate.test(value2));
    }

    /**
     * Creates metadata for the given elytra item.
     *
     * @param elytra <p>The elytra to create original metadata for.</p>
     * @return <p>The generated metadata.</p>
     */
    private OriginalMetadata createElytraMetadata(ItemStack elytra) {
        OriginalMetadata newMetadata = new OriginalMetadata();
        newMetadata.setElytraEnchantments(new EnchantmentContainer(elytra.getEnchantments(), plugin));
        ItemMeta meta = NBTEditor.getOrCreateItemMeta(elytra);
        if (meta.getLore() != null) {
            newMetadata.setElytraLore(meta.getLore());
        }
        newMetadata.setElytraName(meta.getDisplayName());
        return newMetadata;
    }

    /**
     * Creates metadata for the given armor item.
     *
     * @param armor <p>The armor to create original metadata for.</p>
     * @return <p>The generated metadata.</p>
     */
    private OriginalMetadata createArmorMetadata(ItemStack armor) {
        OriginalMetadata newMetadata = new OriginalMetadata();
        newMetadata.setArmorEnchantments(new EnchantmentContainer(armor.getEnchantments(), plugin));
        ItemMeta meta = NBTEditor.getOrCreateItemMeta(armor);
        if (meta.getLore() != null) {
            newMetadata.setArmorLore(meta.getLore());
        }
        newMetadata.setArmorName(meta.getDisplayName());
        return newMetadata;
    }

}
